package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.DataSecurityAuditEnum;
import com.gic.enterprise.constant.DownloadReportSearchTypeEnum;
import com.gic.enterprise.dao.mapper.TabDownloadReportMapper;
import com.gic.enterprise.dto.DownloadReportDTO;
import com.gic.enterprise.entity.TabDownloadReport;
import com.gic.enterprise.qo.DownloadReportQO;
import com.gic.enterprise.service.DownloadReportService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-28 15:52
 */
@Service
public class DownloadReportServiceImpl implements DownloadReportService {

    @Autowired
    private TabDownloadReportMapper tabDownloadReportMapper;

    @Override
    public Integer save(DownloadReportDTO downloadReportDTO) {
        TabDownloadReport tabDownloadReport = EntityUtil.changeEntityByJSON(TabDownloadReport.class, downloadReportDTO);
        tabDownloadReport.setEnterpriseId(downloadReportDTO.getEnterpriseId());
        tabDownloadReport.setReportId(getReportId());
        tabDownloadReport.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        tabDownloadReport.setCreateTime(new Date());
        tabDownloadReportMapper.insertSelective(tabDownloadReport);
        return tabDownloadReport.getDownloadReportId();
    }

    @Override
    public Integer updateDownloadUrl(Integer downloadReportId, String downloadUrl) {
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setDownloadReportId(downloadReportId);
        downloadReport.setDownloadUrl(downloadUrl);
        return tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    @Override
    public Page<TabDownloadReport> listDownloadReport(DownloadReportQO downloadReportQO) {
        PageHelper.startPage(downloadReportQO.getCurrentPage(), downloadReportQO.getPageSize());
//        WAIT(0, "待审核"),
//                PASS(1, "通过"),
//                FAIL(2, "不通过"),
//                OVER_TIME(3, "超时"),
//                BUILD(4, "生成中"),
//                GENERATED(5, "已生成")
        if (downloadReportQO.getSearchType() != null) {
            downloadReportQO.setSearchName(DownloadReportSearchTypeEnum.parseSearchType(downloadReportQO.getSearchType()));
        }
        if (downloadReportQO.getDownloadReportStatus() != null) {
            switch (downloadReportQO.getDownloadReportStatus()) {
                case 0:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.WAIT.getCode());
                    downloadReportQO.setHasDownloadUrl(null);
                    break;
                case 2:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.FAIL.getCode());
                    downloadReportQO.setHasDownloadUrl(null);
                    break;
                case 3:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.WAIT.getCode());
                    downloadReportQO.setAuditExpireTime(new Date());
                    downloadReportQO.setHasDownloadUrl(null);
                    break;
                case 4:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.PASS.getCode());
                    downloadReportQO.setHasDownloadUrl(false);
                    break;
                case 5:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.PASS.getCode());
                    downloadReportQO.setHasDownloadUrl(true);
                    break;
                default:
                    break;
            }
        } else {
            downloadReportQO.setAuditResult(null);
            downloadReportQO.setHasDownloadUrl(null);
        }
        return tabDownloadReportMapper.listDownloadReport(downloadReportQO);
    }

    @Override
    public int updateAudit(Integer downloadReportId, Integer auditResult) {
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setDownloadReportId(downloadReportId);
        downloadReport.setAuditResult(auditResult);
        return tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    @Override
    public TabDownloadReport getById(Integer downloadReportId) {
        return tabDownloadReportMapper.selectByPrimaryKey(downloadReportId);
    }

    @Override
    public void updateAudit(TabDownloadReport downloadReport) {
        tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    private String getReportId() {
        String reportId = CreateRandomUtils.generateNumberStr(8);
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setReportId(reportId);
        TabDownloadReport result = tabDownloadReportMapper.getBySelective(downloadReport);
        if (result == null) {
            return reportId;
        } else {
            return getReportId();
        }
    }
}
