package com.gic.enterprise.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.gic.enterprise.constant.EnterpriseInitCodeEnum;
import com.gic.enterprise.dao.mapper.TabEnterpriseInitMapper;
import com.gic.enterprise.entity.TabEnterpriseInit;
import com.gic.enterprise.service.EnterpriseInitService;

/**
 * @author guojx
 * @date 2019/7/18 11:13 AM
 */
@Service("enterpriseInitService")
public class EnterpriseInitServiceImpl implements EnterpriseInitService {
    @Autowired
    private TabEnterpriseInitMapper tabEnterpriseInitMapper;

    @Override
    public void initByCode(String configCode, Integer enterpriseId) {
        TabEnterpriseInit enterpriseInit = new TabEnterpriseInit();
        enterpriseInit.setEnterpriseId(enterpriseId);
        enterpriseInit.setConfigCode(configCode);
        enterpriseInit.setIsInitConfig(1);
        enterpriseInit.setUpdateTime(new Date());
        tabEnterpriseInitMapper.updateByEnterpriseIdSelective(enterpriseInit);
    }

    @Override
    public void initById(Integer initId) {
        TabEnterpriseInit enterpriseInit = new TabEnterpriseInit();
        enterpriseInit.setIsInitConfig(1);
        enterpriseInit.setEnterpriseInitId(initId);
        tabEnterpriseInitMapper.updateByPrimaryKeySelective(enterpriseInit);
    }

    @Override
    public TabEnterpriseInit getByCode(String configCode, Integer enterpriseId) {
        return tabEnterpriseInitMapper.getByCode(configCode, enterpriseId);
    }

    @Override
    public List<TabEnterpriseInit> listByParentId(Integer parentId, Integer enterpriseId) {
        List<TabEnterpriseInit> list = tabEnterpriseInitMapper.listByParentId(parentId, enterpriseId);
        if (list == null || list.size() < 1) {
            return new ArrayList<>();
        }
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void initEnterpriseConfigInfo(Integer enterpriseId) {
        tabEnterpriseInitMapper.deleteByEnterpriseId(enterpriseId);
        //
        tabEnterpriseInitMapper.initEnterpriseConfigInfo(enterpriseId);

        List<String> storeConfigList = EnterpriseInitCodeEnum
                .getListByParentCode(EnterpriseInitCodeEnum.COMPLETE_STORE_CONFIG.getCode());
        tabEnterpriseInitMapper.updateParentIdBatch(enterpriseId,
                EnterpriseInitCodeEnum.COMPLETE_STORE_CONFIG.getCode(), storeConfigList);

        List<String> memberCardConfigList = EnterpriseInitCodeEnum
                .getListByParentCode(EnterpriseInitCodeEnum.COMPLETE_MEMBER_CARD_CONFIG.getCode());
        tabEnterpriseInitMapper.updateParentIdBatch(enterpriseId,
                EnterpriseInitCodeEnum.COMPLETE_MEMBER_CARD_CONFIG.getCode(), memberCardConfigList);

        List<String> serviceNumberList = EnterpriseInitCodeEnum
                .getListByParentCode(EnterpriseInitCodeEnum.COMPLETE_SERVICE_NUMBER_CONFIG.getCode());
        tabEnterpriseInitMapper.updateParentIdBatch(enterpriseId,
                EnterpriseInitCodeEnum.COMPLETE_SERVICE_NUMBER_CONFIG.getCode(), serviceNumberList);

        List<String> weChatPageList = EnterpriseInitCodeEnum
                .getListByParentCode(EnterpriseInitCodeEnum.COMPLETE_WECHAT_CONFIG.getCode());
        tabEnterpriseInitMapper.updateParentIdBatch(enterpriseId,
                EnterpriseInitCodeEnum.COMPLETE_WECHAT_CONFIG.getCode(), weChatPageList);

        List<String> dataCountConfigList = EnterpriseInitCodeEnum
                .getListByParentCode(EnterpriseInitCodeEnum.COMPLETE_DATA_COUNT_CONFIG.getCode());
        tabEnterpriseInitMapper.updateParentIdBatch(enterpriseId,
                EnterpriseInitCodeEnum.COMPLETE_DATA_COUNT_CONFIG.getCode(), dataCountConfigList);
    }

    @Override
    public TabEnterpriseInit getById(Integer initId) {
        return tabEnterpriseInitMapper.selectByPrimaryKey(initId);
    }

    @Override
    public void unInit(String configCode, Integer enterpriseId) {
        TabEnterpriseInit enterpriseInit = new TabEnterpriseInit();
        enterpriseInit.setEnterpriseId(enterpriseId);
        enterpriseInit.setConfigCode(configCode);
        enterpriseInit.setIsInitConfig(0);
        enterpriseInit.setUpdateTime(new Date());
        tabEnterpriseInitMapper.updateByEnterpriseIdSelective(enterpriseInit);
    }

}
