package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.applet.WeChatMenuTypeEnum;
import com.gic.enterprise.dto.AppletWechatMenuDTO;
import com.gic.enterprise.dto.WeChatMenuDTO;
import com.gic.enterprise.entity.TabAppletWechatMenu;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.AppletWechatMenuApiService;
import com.gic.enterprise.service.AppletWechatMenuService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("appletWechatMenuApiService")
public class AppletWechatMenuApiServiceImpl implements AppletWechatMenuApiService {
    private static final Logger LOGGER = LogManager.getLogger(AppletWechatMenuApiServiceImpl.class);
    @Autowired
    private AppletWechatMenuService appletWechatMenuService;

    @Override
    public ServiceResponse<Integer> save(AppletWechatMenuDTO dto) throws UnsupportedEncodingException {
        //参数校验
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        //是否是主菜单
        boolean isMainMenu = dto.getParentMenuId() == null || dto.getParentMenuId() == 0;
        String menuName = dto.getMenuName();
        boolean isNotRightMenuName;
        //菜单标题，不超过16个字节，子菜单不超过60个字节
        if (isMainMenu) {
            isNotRightMenuName = StringUtils.isBlank(menuName) || menuName.getBytes("UTF-8").length > 16;
        } else {
            isNotRightMenuName = StringUtils.isBlank(menuName) || menuName.getBytes("UTF-8").length > 60;
        }
        if (isNotRightMenuName) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单名称为空或者过长");
        }
        if (!WeChatMenuTypeEnum.isRightCode(dto.getMenuType())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单内容错误");
        }
        if (WeChatMenuTypeEnum.isNeedMenuCode(dto.getMenuType()) && StringUtils.isBlank(dto.getMenuCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "链接配置不能为空");
        }
        if (WeChatMenuTypeEnum.OUT_LINK.getCode() == dto.getMenuType().intValue()) {
            if (StringUtils.isBlank(dto.getMenuUrl())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "外链网址不能为空");
            }
        }
        if (WeChatMenuTypeEnum.APPLET_LINK.getCode() == dto.getMenuType().intValue()
                && StringUtils.isBlank(dto.getMenuUrlPara())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "小程序appId不能为空");
        }
        //todo edit
        //暂时只能外链和小程序
        if (dto.getMenuType().intValue() != WeChatMenuTypeEnum.OUT_LINK.getCode()
                && dto.getMenuType().intValue() != WeChatMenuTypeEnum.APPLET_LINK.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "一期只能选择"
                    + WeChatMenuTypeEnum.OUT_LINK.getMessage() + "和" + WeChatMenuTypeEnum.APPLET_LINK.getMessage());
        }
        if (StringUtils.isBlank(dto.getAppKey())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "公众号key不能为空");
        }
        if (appletWechatMenuService.isRepeatName(null, dto.getMenuName(), dto.getEnterpriseId(), dto.getAppKey())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单名称重复");
        }
        //填充字段值
        if (isMainMenu) {
            //主菜单父级ID统一0
            dto.setParentMenuId(0);
        } else {
            TabAppletWechatMenu parentObject = appletWechatMenuService.getById(dto.getParentMenuId());
            if (parentObject == null || parentObject.getParentMenuId().intValue() != 0) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级菜单ID错误");
            }
        }
        //主菜单不能超过3个，自菜单不超过5个
        List<TabAppletWechatMenu> listByParentId = appletWechatMenuService.listByParentId(dto.getEnterpriseId(),
                dto.getParentMenuId(), dto.getAppKey());
        if (isMainMenu) {
            if (CollectionUtils.isNotEmpty(listByParentId) && listByParentId.size() >= 3) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主菜单不能超过3个");
            }
        } else {
            if (CollectionUtils.isNotEmpty(listByParentId) && listByParentId.size() >= 5) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "子菜单不能超过5个");
            }
        }
        //sort
        if (CollectionUtils.isNotEmpty(listByParentId)) {
            dto.setSort(listByParentId.get(listByParentId.size() - 1).getSort() + 1);
        } else {
            dto.setSort(1);
        }

        Integer menuId = appletWechatMenuService.save(dto);
        return ServiceResponse.success(menuId);
    }

    @Override
    public ServiceResponse<String> edit(AppletWechatMenuDTO dto) throws Exception {
        //参数校验
        TabAppletWechatMenu record = appletWechatMenuService.getById(dto.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单ID错误");
        }
        boolean isMainMenu = record.getParentMenuId().intValue() == 0;
        String menuName = dto.getMenuName();
        boolean isNotRightMenuName;
        //菜单标题，不超过16个字节，子菜单不超过60个字节
        if (isMainMenu) {
            isNotRightMenuName = StringUtils.isBlank(menuName) || menuName.getBytes("UTF-8").length > 16;
        } else {
            isNotRightMenuName = StringUtils.isBlank(menuName) || menuName.getBytes("UTF-8").length > 60;
        }
        if (isNotRightMenuName) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单名称为空或者过长");
        }
        if (!WeChatMenuTypeEnum.isRightCode(dto.getMenuType())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单内容错误");
        }
        if (WeChatMenuTypeEnum.isNeedMenuCode(dto.getMenuType()) && StringUtils.isBlank(dto.getMenuCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "链接配置不能为空");
        }
        if (WeChatMenuTypeEnum.OUT_LINK.getCode() == dto.getMenuType().intValue()) {
            if (StringUtils.isBlank(dto.getMenuUrl())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "外链网址不能为空");
            }
        }
        if (WeChatMenuTypeEnum.APPLET_LINK.getCode() == dto.getMenuType().intValue()
                && StringUtils.isBlank(dto.getMenuUrlPara())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "小程序appId不能为空");
        }
        //todo edit
        //暂时只能外链和小程序
        if (dto.getMenuType().intValue() != WeChatMenuTypeEnum.OUT_LINK.getCode()
                && dto.getMenuType().intValue() != WeChatMenuTypeEnum.APPLET_LINK.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "一期只能选择"
                    + WeChatMenuTypeEnum.OUT_LINK.getMessage() + "和" + WeChatMenuTypeEnum.APPLET_LINK.getMessage());
        }
        if (appletWechatMenuService.isRepeatName(dto.getMenuId(), dto.getMenuName(), record.getEnterpriseId(),
                record.getAppKey())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单名称重复");
        }

        //不能设置的值
        dto.setParentMenuId(record.getParentMenuId());
        dto.setEnterpriseId(record.getEnterpriseId());
        dto.setAppKey(record.getAppKey());
        dto.setSort(record.getSort());

        appletWechatMenuService.update(dto);
        return ServiceResponse.success(dto.getMenuName());
    }

    @Override
    public ServiceResponse<String> deleteByMenuId(Integer menuId) {
        TabAppletWechatMenu record = appletWechatMenuService.getById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单ID错误");
        }

        //同时修改其他的顺序号
        int currentSort = record.getSort();
        List<TabAppletWechatMenu> listByParentId = appletWechatMenuService.listByParentId(record.getEnterpriseId(),
                record.getParentMenuId(), record.getAppKey());
        if (CollectionUtils.isNotEmpty(listByParentId)) {
            //如果有子菜单，不允许删除
            if (record.getParentMenuId().intValue() == 0) {
                //第一层级，需要判断是否有子菜单
                List<TabAppletWechatMenu> childList = appletWechatMenuService.listByParentId(record.getEnterpriseId(),
                        record.getMenuId(), record.getAppKey());
                if (CollectionUtils.isNotEmpty(childList)) {
                    if (childList.size() > 0) {
                        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不能删除，有子菜单");
                    }
                }
            }
            for (TabAppletWechatMenu menu : listByParentId) {
                if (menu.getSort().intValue() > currentSort) {
                    appletWechatMenuService.sort(menu.getMenuId(), menu.getSort().intValue() - 1);
                }
            }
        }
        appletWechatMenuService.deleteByMenuId(menuId);
        return ServiceResponse.success(record.getMenuName());
    }

    @Override
    public ServiceResponse<String> sort(Integer menuId, Integer sort) {
        TabAppletWechatMenu record = appletWechatMenuService.getById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单ID错误");
        }
        //排序
        int currentSort = record.getSort();
        if (sort.intValue() == currentSort) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已在第" + sort + "位置");
        }
        //是否是上移
        boolean isUp = sort.intValue() < currentSort;

        List<TabAppletWechatMenu> listByParentId = appletWechatMenuService.listByParentId(record.getEnterpriseId(),
                record.getParentMenuId(), record.getAppKey());
        if (CollectionUtils.isNotEmpty(listByParentId)) {
            boolean isRightSort = sort == null || sort.intValue() < 1 || sort.intValue() > listByParentId.size();
            if (!isRightSort) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "排序值错误");
            }
            for (TabAppletWechatMenu menu : listByParentId) {
                int tempSort = menu.getSort().intValue();
                if (isUp) {
                    if (tempSort >= sort && tempSort < currentSort) {
                        appletWechatMenuService.sort(menu.getMenuId(), tempSort + 1);
                    }
                } else {
                    if (tempSort > currentSort && tempSort < sort) {
                        appletWechatMenuService.sort(menu.getMenuId(), tempSort - 1);
                    }
                }
            }
        }
        appletWechatMenuService.sort(menuId, sort);
        return ServiceResponse.success(record.getMenuName() + "排序到" + sort);
    }

    @Override
    public ServiceResponse<List<AppletWechatMenuDTO>> listAll(Integer enterpriseId, String appKey) {
        List<TabAppletWechatMenu> list = appletWechatMenuService.listAll(enterpriseId, appKey);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(AppletWechatMenuDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<String> getWeChatMenuJson(Integer enterpriseId, String appKey) {
        //获取主菜单列表
        List<TabAppletWechatMenu> listByParentId = appletWechatMenuService.listByParentId(enterpriseId, 0, appKey);
        if (CollectionUtils.isNotEmpty(listByParentId)) {
            List<WeChatMenuDTO> result = new ArrayList<>();
            for (TabAppletWechatMenu wechatMenu : listByParentId) {
                WeChatMenuDTO parentButton = new WeChatMenuDTO();
                parentButton.setName(wechatMenu.getMenuName());
                //子菜单
                List<TabAppletWechatMenu> childList = appletWechatMenuService.listByParentId(enterpriseId,
                        wechatMenu.getMenuId(), appKey);
                if (CollectionUtils.isNotEmpty(childList)) {
                    List<WeChatMenuDTO> childButtonList = new ArrayList<>();
                    for (TabAppletWechatMenu child : childList) {
                        WeChatMenuDTO childButton = new WeChatMenuDTO();
                        childButton.setName(child.getMenuName());
                        setMenuButtonValue(child, childButton);
                        childButtonList.add(childButton);
                    }
                    parentButton.setSub_button(childButtonList);

                } else {
                    setMenuButtonValue(wechatMenu, parentButton);
                }
                result.add(parentButton);
            }
            Map<String, List<WeChatMenuDTO>> map = new HashMap<>(4);
            map.put("button", result);
            String param = JSONObject.toJSONString(map);
            //https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Creating_Custom-Defined_Menu.html
            //微信自定义菜单文档url
            return ServiceResponse.success(param);
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有菜单数据");
        }
    }

    @Override
    public ServiceResponse<AppletWechatMenuDTO> getDetail(Integer menuId) {
        TabAppletWechatMenu record = appletWechatMenuService.getById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单ID错误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(AppletWechatMenuDTO.class, record));
    }

    private void setMenuButtonValue(TabAppletWechatMenu menu, WeChatMenuDTO button) {
        int menuType = menu.getMenuType().intValue();
        switch (WeChatMenuTypeEnum.getByCode(menuType)) {
            case MESSAGE:
                //todo
                button.setType("click");
                button.setKey(menu.getMenuId().toString());
                break;
            case OUT_LINK:
                button.setType("view");
                button.setUrl(menu.getMenuUrl());
                break;
            case INSIDE_LINK:
                //todo
                break;
            case APPLET_LINK:
                button.setType("miniprogram");
                //错误跳转页面
                button.setUrl("www.baidu.com");
                button.setAppid(menu.getMenuUrlPara());
                LOGGER.info("小程序类型的menuCode:{}", menu.getMenuCode());
                //todo 从menuCode中解析
                button.setPagepath("test");
            default:
        }
    }
}
