package com.gic.enterprise.service.outer.impl;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.billing.SmsPackageStatusEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.entity.TabBillingEnterpriseSmsPackage;
import com.gic.enterprise.service.EnterpriseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.BillingEnterpriseSmsPackageDTO;
import com.gic.enterprise.service.BillingEnterpriseSmsPackageApiService;
import com.gic.enterprise.service.BillingEnterpriseSmsPackageService;

@Service("billingEnterpriseSmsPackageApiService")
public class BillingEnterpriseSmsPackageApiServiceImpl implements BillingEnterpriseSmsPackageApiService {
    @Autowired
    private BillingEnterpriseSmsPackageService billingEnterpriseSmsPackageService;
    @Autowired
    private EnterpriseService enterpriseService;

    @Override
    public ServiceResponse<List<BillingEnterpriseSmsPackageDTO>> listSmsPackage(Integer enterpriseId) {
        return ServiceResponse.success(billingEnterpriseSmsPackageService.listSmsPackage(enterpriseId));
    }

    @Override
    public ServiceResponse<BillingEnterpriseSmsPackageDTO> getEnterpriseSmsPackageDetail(Integer id) {
        TabBillingEnterpriseSmsPackage smsPackage = this.billingEnterpriseSmsPackageService.getSmsPackageDetail(id);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(BillingEnterpriseSmsPackageDTO.class, smsPackage));
    }

    @Override
    public ServiceResponse<Void> monitorStatus(String param) {
        //查询所有的商户
        List<EnterpriseDTO> enterpriseDTOList = enterpriseService.listEnterprise();
        for (EnterpriseDTO enterpriseDTO : enterpriseDTOList) {
            Integer enterpriseId = enterpriseDTO.getEnterpriseId();
            dealWithMonitorStatus(enterpriseId);
        }
        return ServiceResponse.success();
    }

    private void dealWithMonitorStatus(Integer enterpriseId) {
        List<BillingEnterpriseSmsPackageDTO> smsPackageDTOList = billingEnterpriseSmsPackageService
                .listSmsPackage(enterpriseId);
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String today = dateFormat.format(new Date());
        for (BillingEnterpriseSmsPackageDTO sms : smsPackageDTOList) {
            String validDate = dateFormat.format(sms.getEffectiveDate());
            boolean isExpired = validDate.compareTo(today) < 0;
            if (isExpired) {
                //已过期
                sms.setStatus(SmsPackageStatusEnum.EXPIRED.getCode());
                billingEnterpriseSmsPackageService.update(sms);
            } else {
                if (sms.getRemainAccount().intValue() == 0) {
                    //无余量
                    sms.setStatus(SmsPackageStatusEnum.NO_MARGIN.getCode());
                    billingEnterpriseSmsPackageService.update(sms);
                } else  {
                    if (sms.getRemainAccount().intValue() == sms.getTotalAcount().intValue()) {
                        //待使用
                        sms.setStatus(SmsPackageStatusEnum.TO_BE_USE.getCode());
                        billingEnterpriseSmsPackageService.update(sms);
                    } else {
                        //使用中
                        sms.setStatus(SmsPackageStatusEnum.USING.getCode());
                        billingEnterpriseSmsPackageService.update(sms);
                    }
                }
            }

        }
    }
}
