package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.*;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.entity.TabBillingCouponCard;
import com.gic.enterprise.entity.TabBillingOrder;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.SignUtils;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import com.github.pagehelper.PageHelper;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

@Service("billingOrderApiService")
public class BillingOrderApiServcImpl implements BillingOrderApiService {

    private Logger logger = LogManager.getLogger(BillingOrderApiServcImpl.class);
    @Autowired
    private BillingOrderService billingOrderService;
    @Autowired
    private DictApiService dictApiService;
    @Autowired
    private BillingCouponCardService billingCouponCardService;
    @Autowired
    private BillingPayInfoApiService billingPayInfoApiService;
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private BillingSmsPackageRecordService billingSmsPackageRecordService;


    @Override
    @Transactional
    public ServiceResponse<Integer> saveSmsOrderBuyNow(BillingOrderDTO billingOrderDTO, SmsPackageOrderItemDTO smsPackageOrderItemDTO) {
        ServiceResponse<DictDTO> dictResponse = dictApiService.getDict(Constants.SMS_PACKAGE, smsPackageOrderItemDTO.getCode());
        if (!dictResponse.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "选择的套餐包不存在");
        }
        DictDTO dictDTO = dictResponse.getResult();
        smsPackageOrderItemDTO.setSmsAmount(Integer.valueOf(dictDTO.getKey()));
        smsPackageOrderItemDTO.setCost(Double.parseDouble(dictDTO.getContent()));
        Calendar now = Calendar.getInstance();
        now.setTime(new Date());
        now.add(Calendar.MONTH, smsPackageOrderItemDTO.getPackageLength());
        smsPackageOrderItemDTO.setSmsPackageName(billingOrderDTO.getOrderContent());
        try {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            smsPackageOrderItemDTO.setEffectiveDate(simpleDateFormat.parse(simpleDateFormat.format(now.getTime())));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        billingOrderDTO.setItemJson(JSON.toJSONString(smsPackageOrderItemDTO));
        billingOrderDTO.setTotalFee(smsPackageOrderItemDTO.getSmsAmount() * smsPackageOrderItemDTO.getBuyCount() * smsPackageOrderItemDTO.getCost());
        // 判断优惠券是否存在
        Double content = 0d;
        TabBillingCouponCard couponCard = billingCouponCardService.getNotExpiredCouponCardById(billingOrderDTO.getCouponCardId());
        if(billingOrderDTO.getCouponCardId() != null){
            if (couponCard == null) {
                return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "优惠券不存在或已过期");
            }
            content = couponCard.getContent();
        }
        double totalFeePaid = billingOrderDTO.getTotalFee() - content;
        billingOrderDTO.setTotalFeePaid(totalFeePaid > 0 ? totalFeePaid : 0);
        Integer id = billingOrderService.saveOrderBuyNow(billingOrderDTO);
        if(id > 0){
            if(totalFeePaid <= 0){
                this.billingOrderService.updateBySerialNumber(billingOrderDTO.getSerialNumber(), totalFeePaid, BillingOrderStatusEnum.FINISH.getCode(), "");
            }
            this.billingCouponCardService.useCoupCard(billingOrderDTO.getCouponCardId());
        }
        return ServiceResponse.success(id);
    }

    @Override
    @Transactional
    public ServiceResponse<BillingPayInfoDTO> paySmsPackageOrder(Integer enterpriseId, Integer orderId, Integer payType, String authCode){
        TabBillingOrder order = this.billingOrderService.getOrderById(orderId);
        OutPayDTO outPayDTO = new OutPayDTO();
        outPayDTO.setEnterpriseId(enterpriseId);
        outPayDTO.setTotalFee(order.getTotalFee());
        outPayDTO.setTotalFeePaid(order.getTotalFeePaid());
        outPayDTO.setProductDescription(order.getOrderContent());
        ServiceResponse<UserDTO> userByEnterpriseId = this.userApiService.getUserByEnterpriseId(enterpriseId);
        if(userByEnterpriseId.isSuccess()){
            UserDTO result = userByEnterpriseId.getResult();
            if(result != null){
                outPayDTO.setPhone(result.getPhoneNumber());
            }
        }
        outPayDTO.setBuyType(BuyTypeEnum.BUY_SHORT_MESSAGE_PACKAGE.getCode());
        outPayDTO.setTimeStart(new SimpleDateFormat("YYYYMMddHHmmss").format(new Date()));
        outPayDTO.setOrderSerialNumber(order.getSerialNumber());
        outPayDTO.setPayType(payType);
        outPayDTO.setAuthCode(authCode);
        outPayDTO.setSign(SignUtils.getSign(outPayDTO));
        ServiceResponse<BillingPayInfoDTO> response = this.billingPayInfoApiService.savePrePayInfo(outPayDTO);
        if(response.isSuccess()){
            Integer payInfoId = response.getResult().getPayInfoId();
            boolean updatePayInfoId = this.billingOrderService.updateOrderPayInfoId(orderId, payInfoId, payType);
            if(updatePayInfoId){
                order.setPayInfoId(payInfoId);
                if(PayTypeEnum.OFFLINE_PAY.getCode() == payType || PayTypeEnum.BALANCE_PAY.getCode() == payType){

                    ServiceResponse<BillingPayInfoDTO> payResponse = this.billingPayInfoApiService.confirmPay(outPayDTO);
                    if(!payResponse.isSuccess()){
                        return ServiceResponse.failure(payResponse.getCode(), payResponse.getMessage());
                    }
                    /**支付成功更新套餐包**/
                    if(PayTypeEnum.BALANCE_PAY.getCode() == payType){
                        this.billingOrderService.updateBySerialNumber(order.getSerialNumber(), order.getTotalFeePaid(), BillingOrderStatusEnum.FINISH.getCode(), "");
                        saveSmsPackageRecord(outPayDTO);
                    }
                }
            } else {
                return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "更新预付单号失败");
            }
            return ServiceResponse.success(response.getResult());
        }else {
            return EnterpriseServiceResponse.failure(response.getCode(), response.getMessage());
        }
    }

    private void saveSmsPackageRecord(OutPayDTO outPayDTO) {
        BillingOrderDTO orderDTO = this.getOrderBySerialNumber(outPayDTO.getOrderSerialNumber()).getResult();
        billingSmsPackageRecordService.saveSmsPackageRecord(orderDTO);
    }

    @Override
    public ServiceResponse<Page<BillingOrderDTO>> listOrder(String startTime, String endTime, Integer orderStatus,
                                                            String search, int pageNum, int pageSize, Integer enterpriseId) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabBillingOrder> page = this.billingOrderService.listOrder(enterpriseId, orderStatus, search, startTime, endTime);
        Page<BillingOrderDTO> result = PageHelperUtils.changePageHelperToCurrentPage(page, BillingOrderDTO.class);
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<BillingOrderDTO> getOrderById(Integer orderId) {
        TabBillingOrder order = this.billingOrderService.getOrderById(orderId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(BillingOrderDTO.class, order));
    }

    @Override
    public ServiceResponse<BillingOrderDTO> getOrderBySerialNumber(String serialNumber) {
        TabBillingOrder tabOrder = this.billingOrderService.getOrderBySerialNumber(serialNumber);
        BillingOrderDTO orderDTO = EntityUtil.changeEntityByOrika(BillingOrderDTO.class, tabOrder);
        TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(orderDTO.getSerialNumber(), tabOrder.getPayType());
        if (payInfo != null) {
            orderDTO.setPayInfoSerialNumber(payInfo.getSerialNumber());
        }
        if (orderDTO.getCouponCardId() != null) {
            TabBillingCouponCard coupCardDetail = billingCouponCardService.getCoupCardDetail(orderDTO.getCouponCardId());
            orderDTO.setCouponCardContent(coupCardDetail.getContent());
            orderDTO.setCouponCardName(coupCardDetail.getCouponCardName());
        }
        return ServiceResponse.success(orderDTO);
    }

    @Override
    public ServiceResponse<Integer> cancelOrderApproval(Integer orderId) {
        TabBillingOrder order = this.billingOrderService.getOrderById(orderId);
        if (order == null) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
        transferAccountsApprovalApiService.cancelApproval(order.getSerialNumber());
        Integer line = billingOrderService.cancelOrderApproval(orderId, BillingAuditStatusEnum.CANCEL.getCode());
        return ServiceResponse.success(line);
    }

}
