package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.bizdict.api.dto.BizdictDTO;
import com.gic.bizdict.api.service.BizdictService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.rule.*;
import com.gic.enterprise.entity.TabConfigRule;
import com.gic.enterprise.entity.TabConfigRuleEnterprise;
import com.gic.enterprise.entity.TabConfigRuleEnterpriseRel;
import com.gic.enterprise.entity.TabConfigRuleRel;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.ConfigRuleApiService;
import com.gic.enterprise.service.ConfigRuleEnterpriseService;
import com.gic.enterprise.service.ConfigRuleService;
import com.gic.enterprise.service.EnterpriseService;
import com.github.pagehelper.Page;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service("configRuleApiService")
public class ConfigRuleApiServiceImpl implements ConfigRuleApiService {
    @Autowired
    private ConfigRuleService configRuleService;
    @Autowired
    private ConfigRuleEnterpriseService configRuleEnterpriseService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private BizdictService bizdictService;

    @Override
    public ServiceResponse<Integer> saveRule(RuleDTO dto) {
        if (StringUtils.isBlank(dto.getRuleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code不能为空");
        }
        //校验应用code是否存在
        if (!getAppList().containsKey(dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code不存在");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(null, 1, dto.getRuleName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        //应用code唯一性校验
        if (configRuleService.isRepeatAppCode(null, dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code重复");
        }
        //填充
        dto.setAppName(getAppList().get(dto.getAppCode()));
        dto.setRuleName(dto.getRuleName());
        dto.setParentRuleId(0);
        dto.setRuleLevel(1);
        Integer ruleId = configRuleService.saveRule(dto);
        //更新操作ID链数据
        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setRuleChain("_0_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<Integer> editRule(RuleDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
        }
        if (record.getRuleLevel() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
        }
        if (StringUtils.isBlank(dto.getRuleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code不能为空");
        }
        //校验应用code是否存在
        if (!getAppList().containsKey(dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code不存在");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(dto.getRuleId(), 1, dto.getRuleName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        //应用code唯一性校验
        if (configRuleService.isRepeatAppCode(dto.getRuleId(), dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code重复");
        }
        record.setAppName(getAppList().get(dto.getAppCode()));
        record.setRuleName(dto.getRuleName());
        record.setAppCode(dto.getAppCode());
        record.setAppName(dto.getAppName());
        configRuleService.update(record);
        return ServiceResponse.success(dto.getRuleId());
    }

    @Override
    public ServiceResponse<List<RuleDTO>> listRule(String search) {
        Page<TabConfigRule> page = configRuleService.listRule(search, 1, null, 1, Integer.MAX_VALUE);
        if (page != null) {
            List<TabConfigRule> list = page.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return ServiceResponse.success(EntityUtil.changeEntityListNew(RuleDTO.class, list));
            }
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<com.gic.api.base.commons.Page<RuleClassifyDTO>> listRuleClassify(String search,
            Integer ruleId, Integer pageNum, Integer pageSize) {
        Page<TabConfigRule> page = configRuleService.listRule(search, 2, ruleId, pageNum, pageSize);
        com.gic.api.base.commons.Page resultPage = PageHelperUtils.changePageHelperToCurrentPage(page,
                RuleClassifyDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<com.gic.api.base.commons.Page<RuleClassifySceneDTO>> listRuleClassifyScene(String search,
            Integer ruleId, Integer pageNum, Integer pageSize) {
        Page<TabConfigRule> page = configRuleService.listRule(search, 3, ruleId, pageNum, pageSize);
        com.gic.api.base.commons.Page resultPage = PageHelperUtils.changePageHelperToCurrentPage(page,
                RuleClassifySceneDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Integer> saveRuleClassify(RuleClassifyDTO dto) {
        ServiceResponse valid = validRuleClassify(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        dto.setRuleLevel(2);

        Integer ruleId = configRuleService.saveRuleClassify(dto);
        //更新操作ID链数据
        configRuleService.update(
                new TabConfigRule().setRuleId(ruleId).setRuleChain("_0_" + dto.getParentRuleId() + "_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<String> editRuleClassify(RuleClassifyDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，分类不存在");
        }
        if (record.getRuleLevel() != 2) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，不是分类类型");
        }
        dto.setParentRuleId(record.getParentRuleId());
        ServiceResponse valid = validRuleClassify(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        record.setRuleName(dto.getRuleName());
        record.setClassifyIcon(dto.getClassifyIcon());
        record.setRuleDesc(dto.getRuleDesc());
        record.setRemark(dto.getRemark());
        configRuleService.update(record);
        return ServiceResponse.success(record.getRuleName());
    }

    @Override
    public ServiceResponse<Integer> saveRuleClassifyScene(RuleClassifySceneDTO dto) {
        ServiceResponse valid = validRuleClassifyScene(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }

        //上层分类
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());

        dto.setRuleLevel(3);
        Integer firstRuleId = record.getParentRuleId();
        Integer ruleId = configRuleService.saveRuleClassifyScene(dto);

        //更新操作ID链数据
        configRuleService.update(new TabConfigRule().setRuleId(ruleId)
                .setRuleChain("_0_" + firstRuleId + "_" + dto.getParentRuleId() + "_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<String> editRuleClassifyScene(RuleClassifySceneDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，场景不存在");
        }
        if (record.getRuleLevel() != 3) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，不是场景类型");
        }
        dto.setParentRuleId(record.getParentRuleId());
        ServiceResponse valid = validRuleClassifyScene(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        record.setRuleName(dto.getRuleName());
        record.setRemark(dto.getRemark());
        record.setRuleDesc(dto.getRuleDesc());
        record.setMarketObject(dto.getMarketObject());
        record.setMarketScene(dto.getMarketScene());
        configRuleService.update(record);
        return ServiceResponse.success(record.getRuleName());
    }

    @Override
    public ServiceResponse<String> deleteRule(Integer ruleId) {
        //规则不允许删除
        //        TabConfigRule record = configRuleService.getByRuleId(ruleId);
        //        if (record == null) {
        //            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
        //        }
        //        if (record.getRuleLevel() != 1) {
        //            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
        //        }
        //        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setStatus(0));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> deleteRuleClassify(Integer ruleId) {
        TabConfigRule record = configRuleService.getByRuleId(ruleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，分类不存在");
        }
        if (record.getRuleLevel() != 2) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，不是分类类型");
        }
        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setStatus(0));
        //删除下面的场景
        List<TabConfigRule> list = configRuleService.listRule(null, 3, ruleId);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabConfigRule tabConfigRule : list) {
                configRuleService.update(new TabConfigRule().setRuleId(tabConfigRule.getRuleId()).setStatus(0));
            }
        }
        return ServiceResponse.success(record.getRuleName());
    }

    @Override
    public ServiceResponse<String> deleteRuleClassifyScene(Integer ruleId) {
        TabConfigRule record = configRuleService.getByRuleId(ruleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，场景不存在");
        }
        if (record.getRuleLevel() != 3) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，不是场景类型");
        }
        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setStatus(0));
        return ServiceResponse.success(record.getRuleName());
    }

    @Override
    public ServiceResponse<ConfigRuleDTO> getDetailByRuleId(Integer ruleId) {
        TabConfigRule record = configRuleService.getByRuleId(ruleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "记录不存在");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(ConfigRuleDTO.class, record));
    }

    @Override
    public ServiceResponse<Integer> copyRule(Integer ruleId) {
        TabConfigRule record = configRuleService.getByRuleId(ruleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
        }
        if (record.getRuleLevel() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
        }
        //复制规则
        Integer copyRuleId = configRuleService
                .saveRule(new RuleDTO().setRuleName(record.getRuleName() + "-副本").setRuleLevel(1).setParentRuleId(0));
        //更新操作ID链数据
        configRuleService.update(new TabConfigRule().setRuleId(copyRuleId).setRuleChain("_0_" + copyRuleId + "_"));
        //复制分类
        List<TabConfigRule> classifyList = configRuleService.listRule(null, 2, ruleId);
        if (CollectionUtils.isNotEmpty(classifyList)) {
            for (TabConfigRule tabConfigRule : classifyList) {
                Integer classifyId = configRuleService.saveRuleClassify(new RuleClassifyDTO()
                        .setParentRuleId(copyRuleId).setClassifyIcon(tabConfigRule.getClassifyIcon())
                        .setRemark(tabConfigRule.getRemark()).setRuleDesc(tabConfigRule.getRuleDesc()).setRuleLevel(2)
                        .setRuleName(tabConfigRule.getRuleName()));
                //更新操作ID链数据
                String classifyChain = "_0_" + copyRuleId + "_" + classifyId + "_";
                configRuleService.update(new TabConfigRule().setRuleId(classifyId).setRuleChain(classifyChain));
                //复制场景
                List<TabConfigRule> sceneList = configRuleService.listRule(null, 3, tabConfigRule.getRuleId());
                if (CollectionUtils.isNotEmpty(sceneList)) {
                    for (TabConfigRule scene : sceneList) {
                        //新增场景
                        Integer sceneId = configRuleService.saveRuleClassifyScene(new RuleClassifySceneDTO()
                                .setParentRuleId(classifyId).setMarketObject(scene.getMarketObject())
                                .setMarketScene(scene.getMarketScene()).setRemark(scene.getRemark())
                                .setRuleDesc(scene.getRuleDesc()).setRuleLevel(3).setRuleName(scene.getRuleName()));
                        //更新操作ID链数据
                        configRuleService.update(
                                new TabConfigRule().setRuleId(sceneId).setRuleChain(classifyChain + sceneId + "_"));
                    }
                }
            }
        }

        return ServiceResponse.success(copyRuleId);
    }

    @Override
    public ServiceResponse<List<RuleListDTO>> getRuleTree(String search, Integer enterpriseRuleId) {
        Set<String> ruleRelMap = new HashSet<>();
        //查询已勾选平台规则/分类/场景id
        if (enterpriseRuleId != null) {
            List<TabConfigRuleRel> ruleRelList = configRuleEnterpriseService.listRuleRel(enterpriseRuleId);
            if (CollectionUtils.isNotEmpty(ruleRelList)) {
                for (TabConfigRuleRel ruleRel : ruleRelList) {
                    ruleRelMap.add(ruleRel.getRuleId().toString());
                }
            }
        }

        List<TabConfigRule> list;
        if (StringUtils.isBlank(search)) {
            list = configRuleService.listAll(null);
        } else {
            list = configRuleService.getRuleTreeByName(search);
        }
        //调整数据结构
        if (CollectionUtils.isNotEmpty(list)) {
            List<RuleListDTO> result = new ArrayList<>();
            //规则下面的分类列表
            Map<String, List<RuleClassifyListDTO>> ruleMap = new HashMap<>(16);
            //分类下面的场景列表
            Map<String, List<RuleClassifySceneListDTO>> ruleClassifyMap = new HashMap<>(16);
            for (TabConfigRule rule : list) {
                Boolean select = false;
                if (ruleRelMap.contains(rule.getRuleId().toString())) {
                    select = true;
                }
                //必有值，如果为空，则说明数据错误，需要修正
                int ruleLevel = rule.getRuleLevel();
                if (ruleLevel == 1) {
                    if (StringUtils.isBlank(rule.getAppCode())) {
                        //复制的时候可能造成没有code
                        continue;
                    }
                    result.add(new RuleListDTO().setAppCode(rule.getAppCode()).setAppName(rule.getAppName())
                            .setRuleId(rule.getRuleId()).setRuleName(rule.getRuleName()).setSelect(select));
                } else if (ruleLevel == 2) {
                    RuleClassifyListDTO ruleClassify = new RuleClassifyListDTO().setRuleDesc(rule.getRuleDesc())
                            .setRuleId(rule.getRuleId()).setRuleName(rule.getRuleName()).setRemark(rule.getRemark())
                            .setSelect(select).setClassifyIcon(rule.getClassifyIcon());
                    //分类
                    String ruleIdStr = rule.getParentRuleId().toString();
                    List<RuleClassifyListDTO> temp;
                    if (ruleMap.containsKey(ruleIdStr)) {
                        temp = ruleMap.get(ruleIdStr);
                    } else {
                        temp = new ArrayList<>();
                    }
                    temp.add(ruleClassify);
                    ruleMap.put(ruleIdStr, temp);
                } else if (ruleLevel == 3) {
                    RuleClassifySceneListDTO scene = new RuleClassifySceneListDTO()
                            .setMarketScene(rule.getMarketScene()).setMarketObject(rule.getMarketObject())
                            .setRemark(rule.getRemark()).setRuleDesc(rule.getRuleDesc()).setRuleId(rule.getRuleId())
                            .setRuleName(rule.getRuleName()).setSelect(select);
                    //场景
                    String classifyStr = rule.getParentRuleId().toString();
                    List<RuleClassifySceneListDTO> temp;
                    if (ruleClassifyMap.containsKey(classifyStr)) {
                        temp = ruleClassifyMap.get(classifyStr);
                    } else {
                        temp = new ArrayList<>();
                    }
                    temp.add(scene);
                    ruleClassifyMap.put(classifyStr, temp);
                }
            }
            //装载
            for (RuleListDTO ruleTemp : result) {
                String ruleIdStr = ruleTemp.getRuleId().toString();
                List<RuleClassifyListDTO> ruleClassifyList = ruleMap.get(ruleIdStr);
                if (CollectionUtils.isNotEmpty(ruleClassifyList)) {
                    for (RuleClassifyListDTO classifyTemp : ruleClassifyList) {
                        String classifyStr = classifyTemp.getRuleId().toString();
                        classifyTemp.setRuleClassifySceneList(ruleClassifyMap.get(classifyStr));
                    }
                }
                ruleTemp.setRuleClassifyList(ruleClassifyList);
            }
            return ServiceResponse.success(result);
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<List<RuleClassifyListDTO>> listRuleClassifyByAppCode(String appCode, Integer enterpriseId) {
        TabConfigRule record = configRuleService.getByAppCode(appCode);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "该应用未配置");
        }
        //先查询商户规则
        TabConfigRuleEnterpriseRel ruleRel = configRuleEnterpriseService.getByEnterpriseId(enterpriseId);
        Integer enterpriseRuleId = null;
        if (ruleRel != null) {
            enterpriseRuleId = ruleRel.getEnterpriseRuleId();
        }
        if (enterpriseRuleId == null) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(RuleClassifyListDTO.class,
                    configRuleService.listAll(record.getRuleId())));
        } else {
            ServiceResponse<List<RuleListDTO>> ruleListResult = getRuleTree(null, enterpriseRuleId);
            if (ruleListResult.isSuccess()) {
                List<RuleListDTO> ruleList = ruleListResult.getResult();
                if (CollectionUtils.isNotEmpty(ruleList)) {
                    for (RuleListDTO ruleListDTO : ruleList) {
                        if (ruleListDTO.getAppCode().equals(appCode)) {
                            //勾选中的列表
                            List<RuleClassifyListDTO> temp = ruleListDTO.getRuleClassifyList();
                            List<RuleClassifyListDTO> select = new ArrayList<>();
                            if (CollectionUtils.isNotEmpty(temp)) {
                                for (RuleClassifyListDTO dto : temp) {
                                    if (dto.getSelect()) {
                                        //场景重置
                                        dto.setRuleClassifySceneList(null);
                                        select.add(dto);
                                    }
                                }
                            }
                            return ServiceResponse.success(select);
                        }
                    }
                }
            }
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<List<RuleClassifySceneListDTO>> listRuleSceneByRuleClassifyId(Integer ruleClassifyId,
            String appCode, Integer enterpriseId) {
        TabConfigRule record = configRuleService.getByRuleId(ruleClassifyId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，无此分类");
        }
        //先查询商户规则
        TabConfigRuleEnterpriseRel ruleRel = configRuleEnterpriseService.getByEnterpriseId(enterpriseId);
        Integer enterpriseRuleId = null;
        if (ruleRel != null) {
            enterpriseRuleId = ruleRel.getEnterpriseRuleId();
        }
        if (enterpriseRuleId == null) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(RuleClassifySceneListDTO.class,
                    configRuleService.listAll(ruleClassifyId)));
        } else {
            //查询分类下的商户配置的场景，如果没有勾选，则没有数据
            ServiceResponse<List<RuleListDTO>> ruleListResult = getRuleTree(null, enterpriseRuleId);
            if (ruleListResult.isSuccess()) {
                List<RuleListDTO> ruleList = ruleListResult.getResult();
                if (CollectionUtils.isNotEmpty(ruleList)) {
                    for (RuleListDTO ruleListDTO : ruleList) {
                        //有的规则是复制建立的，没有应用code
                        if (StringUtils.isBlank(ruleListDTO.getAppCode())) {
                            continue;
                        }
                        if (ruleListDTO.getAppCode().equals(appCode)) {
                            List<RuleClassifyListDTO> classifyList = ruleListDTO.getRuleClassifyList();
                            if (CollectionUtils.isNotEmpty(classifyList)) {
                                for (RuleClassifyListDTO classifyListDTO : classifyList) {
                                    if (classifyListDTO.getRuleId().intValue() == ruleClassifyId.intValue()) {
                                        if (!classifyListDTO.getSelect()) {
                                            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，商户规则未配置该分类");
                                        }
                                        //勾选中的，即配置可用的
                                        List<RuleClassifySceneListDTO> temp = classifyListDTO.getRuleClassifySceneList();
                                        List<RuleClassifySceneListDTO> select = new ArrayList<>();
                                        if (CollectionUtils.isNotEmpty(temp)) {
                                            for (RuleClassifySceneListDTO dto : temp) {
                                                if (dto.getSelect()) {
                                                    select.add(dto);
                                                }
                                            }
                                        }
                                        return ServiceResponse.success(select);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<EnterpriseRuleDetailDTO> getEnterpriseRuleDetail(Integer enterpriseRuleId) {
        TabConfigRuleEnterprise record = configRuleEnterpriseService.getByEnterpriseRuleId(enterpriseRuleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则D错误，无此规则");
        }
        String ruleName = record.getRuleName();
        //商户列表
        List<EnterpriseListDTO> enterpriseList = new ArrayList<>();
        Page<RuleEnterpriseDTO> page = configRuleEnterpriseService.listEnterpriseRule(null, enterpriseRuleId, 1,
                Integer.MAX_VALUE);
        if (page != null) {
            List<RuleEnterpriseDTO> ruleEnterpriseList = page.getResult();
            if (CollectionUtils.isNotEmpty(ruleEnterpriseList)) {
                for (RuleEnterpriseDTO ruleEnterpriseDTO : ruleEnterpriseList) {
                    enterpriseList.add(new EnterpriseListDTO().setCompanyName(ruleEnterpriseDTO.getCompanyName())
                            .setEnterpriseId(ruleEnterpriseDTO.getEnterpriseId()).setLogo(ruleEnterpriseDTO.getLogo())
                            .setRuleName(ruleName).setEnterpriseName(ruleEnterpriseDTO.getEnterpriseName()));
                }
            }
        }
        //规则树
        ServiceResponse<List<RuleListDTO>> ruleListResult = getRuleTree(null, enterpriseRuleId);
        return ServiceResponse.success(new EnterpriseRuleDetailDTO().setRuleName(ruleName)
                .setEnterpriseType(record.getEnterpriseType()).setEnterpriseList(enterpriseList)
                .setEnterpriseRuleId(enterpriseRuleId).setRuleList(ruleListResult.getResult()));
    }

    @Override
    public ServiceResponse<List<EnterpriseListDTO>> getEnterpriseList(String search) {
        Map<String, TabConfigRuleEnterpriseRel> enterpriseMap = new HashMap<>(16);
        Map<String, String> enterpriseRuleMap = new HashMap<>(16);

        //商户和规则关联数据，查询商户ID和对应规则ID
        List<TabConfigRuleEnterpriseRel> rel = configRuleEnterpriseService.listEnterprise();
        if (CollectionUtils.isNotEmpty(rel)) {
            for (TabConfigRuleEnterpriseRel ruleEnterpriseRel : rel) {
                enterpriseMap.put(ruleEnterpriseRel.getEnterpriseId().toString(), ruleEnterpriseRel);
            }
        }
        //查询所有的商户规则ID、名称
        List<TabConfigRuleEnterprise> ruleEnterpriseList = configRuleEnterpriseService.listEnterpriseRule();
        if (CollectionUtils.isNotEmpty(ruleEnterpriseList)) {
            for (TabConfigRuleEnterprise ruleEnterpriseDTO : ruleEnterpriseList) {
                enterpriseRuleMap.put(ruleEnterpriseDTO.getEnterpriseRuleId().toString(),
                        ruleEnterpriseDTO.getRuleName());
            }
        }
        List<EnterpriseDTO> enterpriseDTOList = enterpriseService.listEnterprise();

        if (CollectionUtils.isNotEmpty(enterpriseDTOList)) {
            List<EnterpriseListDTO> resultList = new ArrayList<>(enterpriseDTOList.size());
            for (EnterpriseDTO enterpriseDTO : enterpriseDTOList) {
                Integer enterpriseId = enterpriseDTO.getEnterpriseId();
                String ruleName = "默认";
                if (enterpriseMap.containsKey(enterpriseId.toString())) {
                    TabConfigRuleEnterpriseRel tempRel = enterpriseMap.get(enterpriseId.toString());
                    ruleName = enterpriseRuleMap.get(tempRel.getEnterpriseRuleId().toString());
                }
                resultList.add(new EnterpriseListDTO().setRuleName(ruleName).setLogo(enterpriseDTO.getLogo())
                        .setEnterpriseName(enterpriseDTO.getEnterpriseName())
                        .setEnterpriseId(enterpriseDTO.getEnterpriseId())
                        .setCompanyName(enterpriseDTO.getCompanyName()));
            }
            return ServiceResponse.success(resultList);
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<Map<String, String>> getAppListByDict() {
        return ServiceResponse.success(getAppList());
    }

    private ServiceResponse<Void> validRuleClassify(RuleClassifyDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
        }
        if (record.getRuleLevel() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
        }
        if (StringUtils.isBlank(dto.getRuleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类名称不能为空");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(dto.getRuleId(), 2, dto.getRuleName(), dto.getParentRuleId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类名称重复");
        }
        if (StringUtils.isBlank(dto.getClassifyIcon())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类icon不能为空");
        }
        if (StringUtils.isBlank(dto.getRuleDesc())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类描述不能为空");
        }
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> validRuleClassifyScene(RuleClassifySceneDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，分类不存在");
        }
        if (record.getRuleLevel() != 2) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，不是分类类型");
        }
        if (StringUtils.isBlank(dto.getMarketScene())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景code不能为空");
        }
        if (StringUtils.isBlank(dto.getRuleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景名称不能为空");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(dto.getRuleId(), 3, dto.getRuleName(), dto.getParentRuleId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        if (StringUtils.isBlank(dto.getRuleDesc())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景描述不能为空");
        }
        Integer marketObject = dto.getMarketObject();
        boolean isNotRightMarketObject = marketObject == null
                || (marketObject != 1 && marketObject != 2 && marketObject != 3);
        if (isNotRightMarketObject) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "营销对象值非法");
        }
        return ServiceResponse.success();
    }

    /**
     * 应用列表
     * @return
     */
    private Map<String, String> getAppList() {
        Map<String, String> result = new HashMap<>(16);
        List<BizdictDTO> list = bizdictService.listByTypeDict("applicaiton_module", "marketing");
        if (CollectionUtils.isNotEmpty(list)) {
            for (BizdictDTO bizdictDTO : list) {
                result.put(bizdictDTO.getCode(), bizdictDTO.getName());
            }
        }
        return result;

    }
}
