package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dto.AppletAppDTO;
import com.gic.enterprise.dto.CustomGuideDTO;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.GuideTypeDTO;
import com.gic.enterprise.entity.TabCustomGuide;
import com.gic.enterprise.entity.TabGuideType;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service("customGuideApiService")
public class CustomGuideApiServiceImpl implements CustomGuideApiService {
    private static final Logger log = LogManager.getLogger(CustomGuideApiServiceImpl.class);
    @Autowired
    private CustomGuideService customGuideService;
    @Autowired
    private GuideTypeService guideTypeService;
    @Autowired
    private CustomStoreApiService customStoreApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private CustomThemeService customThemeService;
    @Autowired
    private AppletAppApiService appletAppApiService;

    @Override
    public ServiceResponse<Integer> getMaxVersion(Integer enterpriseId, String appid) {
        int maxVersion = this.customGuideService.getMaxVersion(enterpriseId, appid);
        return ServiceResponse.success(maxVersion);
    }

    @Override
    public ServiceResponse<List<CustomGuideDTO>> listGuide(Integer enterpriseId, String appid) {
        int maxVersion = this.customGuideService.getMaxVersion(enterpriseId, appid);
        return this.listGuide(enterpriseId, maxVersion, appid);
    }

    @Override
    public ServiceResponse<List<CustomGuideDTO>> listGuide(Integer enterpriseId, Integer version, String appid) {
        List<TabCustomGuide> list = this.customGuideService.listGuide(enterpriseId, version, appid);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(CustomGuideDTO.class, list));
    }

    @Override
    public ServiceResponse<Void> saveGuide(List<CustomGuideDTO> listGuide, boolean isNeedNewVersion,
                                           Integer enterpriseId, String appid) {
        int maxVersion = this.customGuideService.getMaxVersion(enterpriseId, appid);
        if(CollectionUtils.isNotEmpty(listGuide)){
            for(CustomGuideDTO dto : listGuide){
                TabCustomGuide guide = EntityUtil.changeEntityByJSON(TabCustomGuide.class, dto);
                if(isNeedNewVersion){
                    guide.setGuideId(null);
                    guide.setVersion(maxVersion+1);
                }else {
                    guide.setVersion(maxVersion);
                }
                this.customGuideService.saveGuide(guide);
                this.customStoreApiService.customSettingHasCommpleted(enterpriseId);
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<GuideTypeDTO>> listGuideType() {
        List<TabGuideType> list = this.guideTypeService.listGuideType();
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(GuideTypeDTO.class, list));
    }

    @Override
    public ServiceResponse<Map<String, Object>> getExtJson(Integer enterpriseId, String appid) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        if (enterpriseId == null) {
            return ServiceResponse.success();
        }
        ServiceResponse<EnterpriseDTO> response = this.enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!response.isSuccess() || response.getResult() == null) {
            return ServiceResponse.success();
        }
        int maxVersion = this.customGuideService.getMaxVersion(enterpriseId, appid);
        List<TabCustomGuide> guideList = this.customGuideService.listGuide(enterpriseId, maxVersion, appid);
        if (CollectionUtils.isEmpty(guideList)) {
            log.info("该企业没有设置导航 , 企业id=" + enterpriseId);
            return ServiceResponse.success();
        }
        // **1黑色 2红色*/
        int theme = 2;
        Integer theme1 = this.customThemeService.getTheme(enterpriseId, appid);
        if(theme1 != null){
            theme = theme1;
        }
        String themeStr = "normal";
        if (theme == 1) {
            themeStr = "blackTheme";
        }
        // networkTimeout
        Map<String, Object> networkTimeoutMap = new HashMap<String, Object>();
        networkTimeoutMap.put("request", 20000);
        networkTimeoutMap.put("downloadFile", 20000);

        // tabBar
        List<Map<String, Object>> tabBarList = new ArrayList<>();
        Set<String> pathSet = new HashSet<String>() ;
        for (int i = 0; i < guideList.size(); i++) {
            TabCustomGuide guide = guideList.get(i);
            String title = guide.getTitle();
            Map<String, Object> listMap = new HashMap<String, Object>();
            listMap.put("selectedIconPath", guide.getSelectedIconPath());
            listMap.put("iconPath", guide.getIconPath());
            listMap.put("pagePath", guide.getCustomPage());
            pathSet.add(guide.getCustomPage()) ;
            listMap.put("text", title);
            tabBarList.add(listMap);
        }
        if(pathSet.size() != guideList.size()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "存在相同的路径");
        }
        Map<String, Object> tabBar = new HashMap<String, Object>();
        tabBar.put("color", "#999999");
        tabBar.put("selectedColor", "#dabb7d");
        tabBar.put("backgroundColor", "#ffffff");
        tabBar.put("borderStyle", "black");
        tabBar.put("list", tabBarList);

        // window
        Map<String, Object> windowMap = new HashMap<String, Object>();
        windowMap.put("enablePullDownRefresh", false);
        windowMap.put("backgroundTextStyle", "light");
        windowMap.put("navigationBarBackgroundColor", "#fff");
        windowMap.put("navigationBarTitleText", response.getResult().getEnterpriseName());
        windowMap.put("navigationBarTextStyle", "black");

        Map<String, Object> ext = new HashMap<String, Object>();
        ext.put("name", response.getResult().getEnterpriseName());
        ext.put("appid", appid);
        ext.put("brandName", response.getResult().getEnterpriseName());
        ext.put("theme", themeStr);
        ext.put("tabBar", tabBar);
        ext.put("build", maxVersion);
        //
        List<String> appIdList = new ArrayList<>() ;
        ServiceResponse<Page<AppletAppDTO>> page = this.appletAppApiService.page(null, enterpriseId, 1, 1, 1, 1000);
        if(page.isSuccess() && page.getResult()!= null){
            List<AppletAppDTO> wxaList = page.getResult().getResult();
            if(CollectionUtils.isNotEmpty(wxaList)) {
                for(int i=0;i<wxaList.size();i++) {
                    if(i<=9) {
                        AppletAppDTO item = wxaList.get(i);
                        if(!appid.equals(item.getAppId())) {
                            appIdList.add(appid) ;
                        }
                    }
                }
            }
        }
        log.info("ext文件,enterpriseId={} , appid={}" , enterpriseId, appIdList);
        ext.put("navigateToMiniProgramAppIdList", appIdList) ;
        // return
        returnMap.put("extEnable", true);
        returnMap.put("extAppid", appid);
        returnMap.put("ext", ext);
        returnMap.put("extPages", new HashMap<String, Object>());
        returnMap.put("window", windowMap);
        returnMap.put("tabBar", tabBar);
        returnMap.put("networkTimeout", networkTimeoutMap);
        returnMap.put("navigateToMiniProgramAppIdList", appIdList) ;
        return ServiceResponse.success(returnMap);
    }
}
