package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.EnterpriseInitCodeEnum;
import com.gic.enterprise.dto.EnterpriseInitDTO;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.entity.TabEnterpriseInit;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.service.EnterpriseInitService;
import com.gic.enterprise.service.EnterpriseService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author guojx
 * @date 2019/7/18 11:31 AM
 */
@Service("enterpriseInitApiService")
public class EnterpriseInitApiServiceImpl implements EnterpriseInitApiService{
    private static final Logger LOGGER = LogManager.getLogger(EnterpriseInitApiServiceImpl.class);
    @Autowired
    private EnterpriseInitService enterpriseInitService;
    @Autowired
    private EnterpriseService enterpriseService;


    private ServiceResponse<Void> initByConfigCode(String configCode, Integer enterpriseId) {
        LOGGER.info("{}商户初始化：{}", enterpriseId, configCode);
        if (!EnterpriseInitCodeEnum.isRightCode(configCode)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "初始化配置代码参数有误");
        }
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        enterpriseInitService.initByCode(configCode, enterpriseId);
        //查询当前记录
        TabEnterpriseInit enterpriseInit = enterpriseInitService.getByCode(configCode, enterpriseId);
        if (enterpriseInit == null) {
            LOGGER.info("{}商户没有{}初始化项", enterpriseId, configCode);
            return ServiceResponse.success();
        }
        if (enterpriseInit.getParentId() != 0) {
            //如果不是第一层，需要查询子节点是否都已经初始化
            List<TabEnterpriseInit> listByParentId = enterpriseInitService.listByParentId(enterpriseInit.getParentId(), enterpriseId);
            boolean isChildrenAllInit = true;
            for (TabEnterpriseInit tabEnterpriseInit : listByParentId) {
                if (tabEnterpriseInit.getIsInitConfig() != 1) {
                    isChildrenAllInit = false;
                    break;
                }
            }
            //如果是，则父级节点自动初始化
            if (isChildrenAllInit) {
                enterpriseInitService.initById(enterpriseInit.getParentId());
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> addMemberCardInitConfig(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CREATE_MEMBER_CARD.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> addMemberGradeInitConfig(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CREATE_MEMBER_GRADE.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> addIntegralStrategyInitConfig(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CREATE_INTEGRAL_STRATEGY.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> bindServiceNumberInitConfig(Integer enterpriseId) {
        //暂时为了数据的流程完整
        //模拟 完善自动回复配置
        initByConfigCode(EnterpriseInitCodeEnum.PERFECT_AUTO_REPLY_CONFIG.getCode(), enterpriseId);
        return initByConfigCode(EnterpriseInitCodeEnum.BIND_SERVICE_NUMBER.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> perfectAutoReplyConfig(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.PERFECT_AUTO_REPLY_CONFIG.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> perfectMenuConfig(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.PERFECT_MENU_CONFIG.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> bindWeChat(Integer enterpriseId) {
        //暂时为了数据的流程完整
        //模拟 完善小程序页面装修
        initByConfigCode(EnterpriseInitCodeEnum.PERFECT_WECHAT_PAGE.getCode(), enterpriseId);
        return initByConfigCode(EnterpriseInitCodeEnum.BIND_WECHAT.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> perfectWeChatPage(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.PERFECT_WECHAT_PAGE.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> createStoreRegion(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CREATE_STORE_REGION.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> setStoreStatusAndType(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.SET_STORE_STATUS_AND_TYPE.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> createStoreBrand(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CREATE_STORE_BRAND.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> createStoreStatusStrategy(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CREATE_STORE_STATUS_STRATEGY.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> createBelongStrategy(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CREATE_BELONG_STRATEGY.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> configUserMergeStrategy(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CONFIG_USER_MERGE_STRATEGY.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> configEffectiveOrder(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CONFIG_EFFECTIVE_ORDER.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> configPay(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.CONFIG_PAY.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> shortMessageServiceConfig(Integer enterpriseId) {
        return initByConfigCode(EnterpriseInitCodeEnum.SHORT_MESSAGE_SERVICE_CONFIG.getCode(), enterpriseId);
    }

    @Override
    public ServiceResponse<Void> unInitByConfigCode(String configCode, Integer enterpriseId) {
        if (!EnterpriseInitCodeEnum.isRightCode(configCode)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "初始化配置代码参数有误");
        }
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        enterpriseInitService.unInit(configCode, enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<EnterpriseInitDTO> getByConfigCode(String configCode, Integer enterpriseId) {
        if (!EnterpriseInitCodeEnum.isRightCode(configCode)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "初始化配置代码参数有误");
        }
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        //查询当前记录
        TabEnterpriseInit enterpriseInit = enterpriseInitService.getByCode(configCode, enterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityNew(EnterpriseInitDTO.class, enterpriseInit));
    }

    @Override
    public ServiceResponse<List<EnterpriseInitDTO>> listByEnterpriseId(Integer enterpriseId) {
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        List<TabEnterpriseInit> listByParentId = enterpriseInitService.listByParentId(null, enterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(EnterpriseInitDTO.class, listByParentId));
    }


}
