package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.InternationalSmsRecordDTO;
import com.gic.enterprise.service.BillingAccountService;
import com.gic.enterprise.service.BillingInternationalSmsRecordService;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.service.InternationalSmsRecordApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-09 9:26
 */
@Service("internationalSmsRecordApiService")
public class InternationalSmsRecordApiServiceImpl implements InternationalSmsRecordApiService {

    private Logger logger = LogManager.getLogger(InternationalSmsRecordApiServiceImpl.class);

    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingInternationalSmsRecordService billingInternationalSmsRecordService;
    @Autowired
    private DictApiService dictApiService;

    @Override
    public ServiceResponse<Void> saveInternationalSmsRecord(InternationalSmsRecordDTO internationalSmsRecordDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(internationalSmsRecordDTO, InternationalSmsRecordDTO.InternationalSmsRecordValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            String json = JSON.toJSONString(internationalSmsRecordDTO);
            logger.info("收到海外短信计费信息：{}", json);
            client.sendMessage("InternationalSmsMq", json);
//            this.run(json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {
        // 缓存中计费
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("海外短信缓存扣费：{}", arg);
            InternationalSmsRecordDTO internationalSmsRecordDTO = JSON.parseObject(arg, InternationalSmsRecordDTO.class);
            // 判断是扣套餐包还是扣余额
//            Integer fee = accountStandardService.queryUnitFee(internationalSmsRecordDTO.getEnterpriseId(), AccountStandardEnum.SMS.getCode());
            billingAccountService.deductAccountByCache(internationalSmsRecordDTO.getFee(), internationalSmsRecordDTO.getEnterpriseId());

            // todo 队列
            client.sendMessage("InternationalSmsDbMq", arg);
//            this.executeDatabase(arg);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("海外短信数据库扣费：{}", arg);
        InternationalSmsRecordDTO internationalSmsRecordDTO = JSON.parseObject(arg, InternationalSmsRecordDTO.class);
        internationalSmsRecordDTO.setDeductionTime(new Date());
        ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.SMS_APP_TYPE, internationalSmsRecordDTO.getAppCode());
        if (serviceDict.isSuccess()) {
            DictDTO dictDTO = serviceDict.getResult();
            if (dictDTO != null) {
                String key = dictDTO.getKey();
                internationalSmsRecordDTO.setAppName(key);
            }
        }
        billingAccountService.deductAccountByDatabase(internationalSmsRecordDTO.getFee(), internationalSmsRecordDTO.getEnterpriseId());
        billingInternationalSmsRecordService.save(internationalSmsRecordDTO);
        return ServiceResponse.success();
    }
}
