package com.gic.enterprise.service.outer.impl.pay;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.BillingAuditStatusEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.ApprovalCallBackDTO;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.OutPayDTO;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.BillingOrderService;
import com.gic.enterprise.service.BillingPayInfoService;
import com.gic.enterprise.service.BillingRechargeRecordService;
import com.gic.enterprise.service.PushMessageApiService;
import com.gic.store.utils.CreateSerialNumberUtils;
import com.gic.thirdparty.api.dto.ScanPayReqDataDTO;
import com.gic.thirdparty.api.service.Pay4WXService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/20 15:35
 */
@Component("1")
public class WxPayStrategy implements PayStrategy {
    private Logger logger = LogManager.getLogger(WxPayStrategy.class);

    @Autowired
    private Pay4WXService pay4WXService;
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private BillingRechargeRecordService billingRechargeRecordService;
    @Autowired
    private BillingOrderService billingOrderService;
    @Autowired
    private PushMessageApiService pushMessageApiService;

    @Override
    public ServiceResponse<BillingPayInfoDTO> savePrePayInfo(OutPayDTO outPayDTO) {
        if (StringUtils.isNotBlank(outPayDTO.getOrderSerialNumber())) {
            TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(outPayDTO.getOrderSerialNumber(), outPayDTO.getPayType());
            if (payInfo != null) {
                BillingPayInfoDTO payInfoDTO = EntityUtil.changeEntityByOrika(BillingPayInfoDTO.class, payInfo);
                return ServiceResponse.success(payInfoDTO);
            }
        }

        String serialNumber = CreateSerialNumberUtils.createSerialNumber();
        BillingPayInfoDTO billingPayInfoDTO = saveWxPrePay(outPayDTO.getEnterpriseId(), serialNumber, outPayDTO.getTotalFee(), outPayDTO.getBuyType(), outPayDTO.getOrderSerialNumber());
        // 微信支付
        if (billingPayInfoDTO != null) {
            return ServiceResponse.success(billingPayInfoDTO);
        } else {
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "调用微信支付失败");
        }
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> confirmPay(OutPayDTO outPayDTO) {
        logger.info("线下转账没有该方法 confirmPay");
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Boolean> queryAndUpdatePayStatus(Integer enterpriseId, String serialNumber) {
        TabBillingPayInfo payInfo = billingPayInfoService.getBySerialNumber(serialNumber);
        return ServiceResponse.success(payInfo != null && payInfo.getResultCode() == 1);
//        // 微信支付
//        ServiceResponse<ScanPayQueryResData> serviceResponse = pay4WXService.scanPayOrderQuery(serialNumber);
//        if (serviceResponse.isSuccess()) {
//            ScanPayQueryResData resData = serviceResponse.getResult();
//            if (StringUtils.equalsIgnoreCase(resData.getReturn_code(), "SUCCESS")) {
//                Integer i = billingPayInfoService.updatePayInfoBySerialNumber(
//                        enterpriseId,
//                        serialNumber,
//                        StringUtils.equalsIgnoreCase(resData.getResult_code(), "SUCCESS") ? 1 : 0,
//                        resData.getErr_code(),
//                        resData.getErr_code_des()
//                );
//            }
//            return ServiceResponse.success(StringUtils.equalsIgnoreCase(resData.getResult_code(), "SUCCESS"));
//        } else {
//            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
//        }

    }


    @Override
    public ServiceResponse<Void> callBack(ApprovalCallBackDTO resData) {
        logger.info("微信支付回调结果:{}", JSON.toJSONString(resData));
        // 写入支付结果

        TabBillingPayInfo billingPayInfo = billingPayInfoService.getBySerialNumber(resData.getPaySerialNumber());
        if (StringUtils.isBlank(billingPayInfo.getTimeEnd())) {
            Integer i = billingPayInfoService.updatePayInfoBySerialNumber(resData);
            switch (billingPayInfo.getBuyType()) {
                case 1:
                    // 商户余额充值
                    billingRechargeRecordService.updateBySerialNumber(resData.getEnterpriseId(), billingPayInfo.getOrderSerialNumber(), resData.getTotalFeePaid(), resData.getTotalFeePaid(), BillingAuditStatusEnum.PASS.getCode(), resData.getTimeEnd());
                    this.pushMessageApiService.pushMessage(resData.getEnterpriseId(), "产品通知", "余额充值", "您的商户成功充值"+resData.getTotalFeePaid()+"元！感谢您的支持和使用！");
                    break;
                case 2:
                    // 短信套餐包购买
                    this.billingOrderService.updateBySerialNumber(billingPayInfo.getOrderSerialNumber(), resData.getTotalFeePaid(), BillingAuditStatusEnum.PASS.getCode(), resData.getTimeEnd());
                    break;
                default:
                    break;
            }
            // 删除其他的支付方式
            billingPayInfoService.delOtherPayInfoExceptId(billingPayInfo.getPayInfoId(), billingPayInfo.getEnterpriseId(), billingPayInfo.getOrderSerialNumber());

        } else {
            logger.info("微信重复回调 paySerialNumber:{}", resData.getPaySerialNumber());
        }

        return ServiceResponse.success();
    }

    private BillingPayInfoDTO saveWxPrePay(Integer enterpriseId, String serialNumber, Double totalFee, Integer buyType, String orderSerialNumber) {
        Date startDate = new Date();
        String startDateFormat = DateUtil.formatString(startDate, DateUtil.FORMAT_DATETIME_14);
        // 生成付款码
        ScanPayReqDataDTO scanPayReqDataDTO = new ScanPayReqDataDTO(
                "充值",
                "gic:" + enterpriseId.toString(),
                serialNumber,
                new BigDecimal(totalFee.toString()).multiply(new BigDecimal("100")).intValue(),
                startDateFormat,
                "",
                "NATIVE"
        );
        ServiceResponse<String> serviceResponse = pay4WXService.createPrePaymentOrderForWX(scanPayReqDataDTO);
        if (serviceResponse.isSuccess()) {
            // 生成支付记录
            BillingPayInfoDTO billingPayInfoDTO = new BillingPayInfoDTO();
            billingPayInfoDTO.setEnterpriseId(enterpriseId);
            billingPayInfoDTO.setSerialNumber(serialNumber);
            billingPayInfoDTO.setOrderSerialNumber(orderSerialNumber);
            billingPayInfoDTO.setPayType(PayTypeEnum.WE_CHAT_PAY.getCode());
            billingPayInfoDTO.setTimeStart(startDateFormat);
            billingPayInfoDTO.setCodeUrl(serviceResponse.getResult());
            billingPayInfoDTO.setTotalFeePaid(totalFee);
            billingPayInfoDTO.setInvoiceStatus(GlobalInfo.DATA_STATUS_DELETE);
            billingPayInfoDTO.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
            billingPayInfoDTO.setCreateTime(new Date());
            billingPayInfoDTO.setBuyType(buyType);
            Integer payInfoId = billingPayInfoService.savePayInfo(billingPayInfoDTO);
            billingPayInfoDTO.setPayInfoId(payInfoId);
            return billingPayInfoDTO;
        } else {
            return null;
        }
    }

}
