package com.gic.enterprise.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.wechat.business.api.service.fwh.WeixinMenuFunService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.constant.applet.WeChatMenuTypeEnum;
import com.gic.enterprise.dto.AppletWechatMenuDTO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.AppletWechatMenuApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.WechatMenuVO;

@RestController
@RequestMapping("/wechat-menu")
public class WechatMenuController {
    private static final Logger LOGGER = LogManager.getLogger(WechatMenuController.class);

    @Autowired
    private AppletWechatMenuApiService appletWechatMenuApiService;
    @Autowired
    private WeixinMenuFunService weixinMenuFunService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;

    @RequestMapping("/save")
    public RestResponse saveMenu(AppletWechatMenuDTO dto) throws Exception {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(appletWechatMenuApiService.save(dto),
                OperationResultUtils.LOG_NEW + "自定义菜单", dto.getMenuName());
    }

    @RequestMapping("/edit")
    public RestResponse edit(AppletWechatMenuDTO dto) throws Exception {
        return OperationResultUtils.operationResult(appletWechatMenuApiService.edit(dto),
                OperationResultUtils.LOG_EDIT + "自定义菜单", dto.getMenuName());
    }

    @RequestMapping("/delete")
    public RestResponse delete(Integer menuId) throws Exception {
        ServiceResponse<String> result = appletWechatMenuApiService.deleteByMenuId(menuId);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "自定义菜单",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/sort")
    public RestResponse sort(Integer menuId, Integer sort) throws Exception {
        ServiceResponse<String> result = appletWechatMenuApiService.sort(menuId, sort);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_SORT + "自定义菜单",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/get-detail")
    public RestResponse sort(Integer menuId) throws Exception {
        return ResultControllerUtils.commonResult(appletWechatMenuApiService.getDetail(menuId));
    }

    @RequestMapping("/list")
    public RestResponse list(String appKey) {
        ServiceResponse<List<AppletWechatMenuDTO>> result = appletWechatMenuApiService
                .listAll(UserDetailUtils.getUserDetail().getEnterpriseId(), appKey);
        if (result.isSuccess()) {
            List<WechatMenuVO> list = EntityUtil.changeEntityListNew(WechatMenuVO.class, result.getResult());
            return RestResponse.success(listToTree(0, list));
        }

        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/synchronize-menu")
    public RestResponse synchronizeMenu(String appKey) {
        ServiceResponse<String> result = appletWechatMenuApiService
                .getWeChatMenuJson(UserDetailUtils.getUserDetail().getEnterpriseId(), appKey);
        if (result.isSuccess()) {
            String json = result.getResult();
            LOGGER.info("自定义菜单组装json:{}", json);
            //delete
            //create
            weixinMenuFunService.deleteMenu(appKey);
            ServiceResponse<String> response = weixinMenuFunService.createMenu(appKey, json);
            LOGGER.info("自定义菜单同步创建结果:{}", JSON.toJSONString(response));

            //初始化回调
            enterpriseInitApiService.perfectMenuConfig(UserDetailUtils.getUserDetail().getEnterpriseId());
            //日志
            LogUtils.createLog("自定义菜单同步", appKey);
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/synchronize-menu-test")
    public RestResponse synchronizeMenu() {
        weixinMenuFunService.deleteMenu("wxc6fd9beaf9a9dbe9");
        ServiceResponse<String> response = weixinMenuFunService.createMenu("wxc6fd9beaf9a9dbe9", "{\"button\":[{\"appid\":\"\",\"name\":\"会员中心\",\"pagepath\":\"test\",\"type\":\"miniprogram\",\"url\":\"www.baidu.com\"}]}");
        return RestResponse.success(response);
    }

    @RequestMapping("/list-menu-type")
    public RestResponse listMenuType() {
        List<Map<String, Object>> list = new ArrayList<>();
        Map<String, Object> result;
        for (WeChatMenuTypeEnum type : WeChatMenuTypeEnum.values()) {
            result = new HashMap<>(4);
            result.put("key", type.getCode());
            result.put("value", type.getMessage());
            list.add(result);
        }
        return RestResponse.success(list);
    }

    private static List<WechatMenuVO> listToTree(int parentMenuId, List<WechatMenuVO> list) {
        List<WechatMenuVO> result = null;
        if (CollectionUtils.isNotEmpty(list)) {
            for (WechatMenuVO vo : list) {
                if (vo.getParentMenuId().intValue() == parentMenuId) {
                    vo.setChild(listToTree(vo.getMenuId(), list));
                    if (result == null) {
                        result = new ArrayList<>();
                    }
                    result.add(vo);
                }
            }
        }
        return result;
    }
}
