package com.gic.operation.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gic.enterprise.error.ErrorCode;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.rule.RuleClassifyDTO;
import com.gic.enterprise.dto.rule.RuleClassifySceneDTO;
import com.gic.enterprise.dto.rule.RuleDTO;
import com.gic.enterprise.dto.rule.RuleEnterpriseDTO;
import com.gic.enterprise.service.ConfigRuleApiService;
import com.gic.enterprise.service.ConfigRuleEnterpriseApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.operation.web.vo.SceneCategoryVO;
import com.gic.operation.web.vo.configrule.*;
import com.gic.rule.manage.api.service.DealOperationApiService;

@RestController
@RequestMapping("/config/rule")
public class ConfigRuleController {

    private static final Logger LOGGER = LogManager.getLogger(ConfigRuleController.class);

    @Autowired
    private ConfigRuleApiService configRuleApiService;
    @Autowired
    private DealOperationApiService dealOperationApiService;
    @Autowired
    private ConfigRuleEnterpriseApiService configRuleEnterpriseApiService;

    @RequestMapping("/saveRule")
    public RestResponse saveRule(RuleDTO dto) {
        return ResultControllerUtils.commonResult(configRuleApiService.saveRule(dto));
    }

    @RequestMapping("/get-app-list")
    public RestResponse listApp() {
        return ResultControllerUtils.commonResult(configRuleApiService.getAppListByDict());
    }

    @RequestMapping("/editRule")
    public RestResponse editRule(RuleDTO dto) {
        return ResultControllerUtils.commonResult(configRuleApiService.editRule(dto));
    }

    @RequestMapping("/listRule")
    public RestResponse listRule(String search) {
        return ResultControllerUtils.commonResult(configRuleApiService.listRule(search), RuleVO.class);
    }

    @RequestMapping("/getRule")
    public RestResponse getRule(Integer ruleId) {
        return ResultControllerUtils.commonResultOne(configRuleApiService.getDetailByRuleId(ruleId), RuleVO.class);
    }

    @RequestMapping("/copyRule")
    public RestResponse copyRule(Integer ruleId) {
        return ResultControllerUtils.commonResult(configRuleApiService.copyRule(ruleId));
    }

    @RequestMapping("/saveRuleClassify")
    public RestResponse saveRuleClassify(RuleClassifyDTO dto) {
        return ResultControllerUtils.commonResult(configRuleApiService.saveRuleClassify(dto));
    }

    @RequestMapping("/editRuleClassify")
    public RestResponse editRuleClassify(RuleClassifyDTO dto) {
        return ResultControllerUtils.commonResult(configRuleApiService.editRuleClassify(dto));
    }

    @RequestMapping("/listRuleClassify")
    public RestResponse listRuleClassify(String search, Integer ruleId, Integer pageNum, Integer pageSize) {
        return ResultControllerUtils.commonPageResult(
                configRuleApiService.listRuleClassify(search, ruleId, pageNum, pageSize), RuleClassifyVO.class);
    }

    @RequestMapping("/getRuleClassify")
    public RestResponse getRuleClassify(Integer ruleId) {
        return ResultControllerUtils.commonResultOne(configRuleApiService.getDetailByRuleId(ruleId),
                RuleClassifyVO.class);
    }

    @RequestMapping("/saveRuleClassifyScene")
    public RestResponse saveRuleClassifyScene(RuleClassifySceneDTO dto) {
        return ResultControllerUtils.commonResult(configRuleApiService.saveRuleClassifyScene(dto));
    }

    @RequestMapping("/editRuleClassifyScene")
    public RestResponse editRuleClassifyScene(RuleClassifySceneDTO dto) {
        return ResultControllerUtils.commonResult(configRuleApiService.editRuleClassifyScene(dto));
    }

    @RequestMapping("/listRuleClassifyScene")
    public RestResponse listRuleClassifyScene(String search, Integer ruleId, Integer pageNum, Integer pageSize) {
        return ResultControllerUtils.commonPageResult(
                configRuleApiService.listRuleClassifyScene(search, ruleId, pageNum, pageSize),
                RuleClassifySceneVO.class);
    }

    @RequestMapping("/getRuleClassifyScene")
    public RestResponse getRuleClassifyScene(Integer ruleId) {
        return ResultControllerUtils.commonResultOne(configRuleApiService.getDetailByRuleId(ruleId),
                RuleClassifySceneVO.class);
    }

    @RequestMapping("/deleteClassify")
    public RestResponse deleteClassify(Integer ruleId) {
        return ResultControllerUtils.commonResult(configRuleApiService.deleteRuleClassify(ruleId));
    }

    @RequestMapping("/deleteClassifyScene")
    public RestResponse deleteClassifyScene(Integer ruleId) {
        return ResultControllerUtils.commonResult(configRuleApiService.deleteRuleClassifyScene(ruleId));
    }

    @RequestMapping("/list-scene")
    public RestResponse listScene(String search, Integer pageNum, Integer pageSize) {
        return ResultControllerUtils.commonPageResult(
                dealOperationApiService.pageSceneCategory(search, pageNum, pageSize), SceneCategoryVO.class);
    }

    @RequestMapping("/list-market-object")
    public RestResponse listMarketObject() {
        List<Map<String, String>> result = new ArrayList<>(3);
        Map<String, String> map = new HashMap<>(8);
        map.put("1", "会员卡域");
        map.put("2", "服务号域");
        map.put("3", "小程序域");
        result.add(map);
        return RestResponse.success(result);
    }

    /**
     * 规则树
     * @param search 规则（应用）/分类/场景名称
     * @return
     */
    @RequestMapping("/get-rule-tree")
    public RestResponse getRuleTree(String search, Integer enterpriseRuleId) {
        return ResultControllerUtils.commonResult(configRuleApiService.getRuleTree(search, enterpriseRuleId));
    }

    /**
     * 商户规则新增
     * @param dto
     * @return
     */
    @RequestMapping("/saveEnterpriseRule")
    public RestResponse saveEnterpriseRule(RuleEnterpriseDTO dto) {
        if (dto.getEnterpriseRuleId() != null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID有值");
        }
        return ResultControllerUtils.commonResult(configRuleEnterpriseApiService.save(dto));
    }

    @RequestMapping("/editEnterpriseRule")
    public RestResponse editEnterpriseRule(RuleEnterpriseDTO dto) {
        if (dto.getEnterpriseRuleId() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID不能为空");
        }
        return ResultControllerUtils.commonResult(configRuleEnterpriseApiService.edit(dto));
    }

    @RequestMapping("/getEnterpriseRule")
    public RestResponse getEnterpriseRule(Integer enterpriseRuleId) {
        return ResultControllerUtils.commonResult(configRuleApiService.getEnterpriseRuleDetail(enterpriseRuleId));
    }

    /**
     * 查询可选择配置的商户列表
     * @Title: listEnterpriseSelect 
     * @Description:
     * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list-enterprise-select")
    public RestResponse listEnterpriseSelect(String search) {
        return ResultControllerUtils.commonResult(configRuleApiService.getEnterpriseList(search));
    }

    @RequestMapping("/deleteEnterpriseRule")
    public RestResponse deleteEnterpriseRule(Integer enterpriseRuleId) {
        return ResultControllerUtils
                .commonResult(configRuleEnterpriseApiService.deleteEnterpriseRule(enterpriseRuleId));
    }

    /**
     * 从规则移除商户，因为一个商户只能对应一个规则，所以只需要商户ID参数
     * @param enterpriseId
     * @return
     */
    @RequestMapping("/removeEnterprise")
    public RestResponse removeEnterprise(Integer enterpriseId) {
        return ResultControllerUtils.commonResult(configRuleEnterpriseApiService.removeEnterprise(enterpriseId));
    }

    @RequestMapping("/list-enterprise-rule")
    public RestResponse listEnterpriseRule(String search, Integer pageNum, Integer pageSize) {
        return ResultControllerUtils.commonPageResult(configRuleEnterpriseApiService.listRule(search, pageNum, pageSize),
                EnterpriseRuleVO.class);
    }

    @RequestMapping("/list-enterprise")
    public RestResponse listEnterprise(String search, Integer enterpriseRuleId, Integer pageNum, Integer pageSize) {
        return ResultControllerUtils.commonPageResult(
                configRuleEnterpriseApiService.listEnterpriseRule(search, enterpriseRuleId, pageNum, pageSize),
                EnterpriseVO.class);
    }
}
