package com.gic.operation.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.constant.EnterpriseInitCodeEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseInitDTO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.marketing.api.dto.sms.EnterpriseSmsSignDTO;
import com.gic.marketing.api.dto.sms.SmsApplicationDTO;
import com.gic.operation.web.vo.EnterpriseSmsSignVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.marketing.api.service.sms.SmsOuterApiService;

import java.util.List;

@RestController
@RequestMapping("/enterprise/sms")
public class EnterpriseSmsController {
    private static final Logger LOGGER = LogManager.getLogger(EnterpriseSmsController.class);

    @Autowired
    private SmsOuterApiService smsOuterApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;

    @RequestMapping("/save-app")
    public RestResponse saveApp(Integer enterpriseId, String appid, String appkey) {
        return ResultControllerUtils.commonResult(smsOuterApiService.saveEnterpriseSmsApp(enterpriseId, appid, appkey));
    }

    @RequestMapping("/get-app")
    public RestResponse getApp(Integer enterpriseId) {
        return ResultControllerUtils.commonResult(smsOuterApiService.getEnterpriseSmsApp(enterpriseId));
    }

    @RequestMapping("/list-sms")
    public RestResponse listSms(Integer enterpriseId, Page<EnterpriseSmsSignDTO> page) {
        return ResultControllerUtils.commonPageResult(smsOuterApiService.listSmsSignByEid(enterpriseId, page),
                EnterpriseSmsSignVO.class);
    }

    /**
     * @Title: addSms 
     * @Description: 
      * @author guojuxing 
     * @param enterpriseId
     * @param smsSignId 短信签名id
     * @param smsSignText 短信签名
     * @param useType  签名用途 1: 国内短信; 2: 海外短信
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/add-sms")
    public RestResponse addSms(Integer enterpriseId, String smsSignId, String smsSignText, int useType) {
        ServiceResponse<Void> smsResult = smsOuterApiService.saveEnterpriseSmsSign(enterpriseId, smsSignId, smsSignText,
                useType);
        if (smsResult.isSuccess()) {
            //新增之前判断是否已经初始化完毕
            ServiceResponse<EnterpriseInitDTO> result = enterpriseInitApiService
                    .getByConfigCode(EnterpriseInitCodeEnum.SHORT_MESSAGE_SERVICE_CONFIG.getCode(), enterpriseId);
            if (result.isSuccess()) {
                EnterpriseInitDTO enterpriseInitDTO = result.getResult();
                if (enterpriseInitDTO.getIsInitConfig().intValue() == 0) {
                    //如果是未配置，则需要判断
                    if (isSmsConfig(enterpriseId)) {
                        //短信业务配置初始化
                        enterpriseInitApiService.shortMessageServiceConfig(enterpriseId);
                    }
                }
            }
        }
        return EnterpriseRestResponse.failure(smsResult);
    }

    @RequestMapping("/get-sms")
    public RestResponse getSms(Long signId) {
        return ResultControllerUtils.commonResultOne(smsOuterApiService.getSmsSignByid(signId), EnterpriseSmsSignVO.class);
    }

    @RequestMapping("/edit-sms")
    public RestResponse editSms(Long signId, String smsSignId, String smsSignText, int useType) {
        return ResultControllerUtils
                .commonResult(smsOuterApiService.editEnterpriseSmsSign(signId, smsSignId, smsSignText, useType));
    }

    @RequestMapping("/delete-sms")
    public RestResponse deleteSms(Long signId, Integer enterpriseId) {
        ServiceResponse<Void> smsResult = smsOuterApiService.delEnterpriseSmsSign(signId);
        if (smsResult.isSuccess()) {
            ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
            if (enterpriseResult.isSuccess()) {
                //商户未初始化，才能回退短信业务配置
                if (enterpriseResult.getResult().getIsInitComplete().intValue() == 0) {
                    ServiceResponse<EnterpriseInitDTO> result = enterpriseInitApiService.getByConfigCode(
                            EnterpriseInitCodeEnum.SHORT_MESSAGE_SERVICE_CONFIG.getCode(), enterpriseId);
                    if (result.isSuccess()) {
                        EnterpriseInitDTO enterpriseInitDTO = result.getResult();
                        if (enterpriseInitDTO.getIsInitConfig().intValue() == 1) {
                            //如果是已配置，则需要判断
                            if (!isSmsConfig(enterpriseId)) {
                                //短信业务配置初始化回退
                                enterpriseInitApiService.unInitByConfigCode(
                                        EnterpriseInitCodeEnum.SHORT_MESSAGE_SERVICE_CONFIG.getCode(), enterpriseId);
                            }
                        }
                    }
                }
            }
        }
        return EnterpriseRestResponse.failure(smsResult);
    }

    /**
     * 是否符合短信业务配置条件
     * @param enterpriseId
     * @return
     */
    private boolean isSmsConfig(Integer enterpriseId) {
        //appID和Appkey判断是否已经配置
        ServiceResponse<SmsApplicationDTO> smsAppResult = smsOuterApiService.getEnterpriseSmsApp(enterpriseId);
        if (smsAppResult.isSuccess()) {
            if (smsAppResult.getResult() == null) {
                return false;
            }
        } else {
            return false;
        }

        Page<EnterpriseSmsSignDTO> page = new Page<>();
        page.setCurrentPage(1);
        page.setPageSize(Integer.MAX_VALUE);
        ServiceResponse<Page<EnterpriseSmsSignDTO>> result = smsOuterApiService.listSmsSignByEid(enterpriseId, page);
        if (result.isSuccess()) {
            Page<EnterpriseSmsSignDTO> page1 = result.getResult();
            if (page1 != null) {
                List<EnterpriseSmsSignDTO> list = page1.getResult();
                int chinaCount = 0;
                int foreignCount = 0;
                if (CollectionUtils.isNotEmpty(list)) {
                    for (EnterpriseSmsSignDTO dto : list) {
                        if (dto.getUseType().intValue() == 1) {
                            chinaCount = chinaCount + 1;
                        } else if (dto.getUseType().intValue() == 2) {
                            foreignCount = foreignCount + 1;
                        }
                        if (chinaCount > 0 && foreignCount > 0) {
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }
}
